@@ -7,7 +7,7 @@ use Mail::Box;
 use Mail::Box::Index;
 
 our @ISA     = qw/Mail::Box Mail::Box::Index/;
-our $VERSION = v0.8;
+our $VERSION = v0.9;
 
 use Mail::Box;
 
@@ -361,6 +361,8 @@ sub readMessages()
         my $size    = -s $msgfile;
         undef $head if $head && $head->get('x-mailbox-size') != $size;
 
+        my @labels  = defined $labels[$msgnr] ? @{$labels[$msgnr]} : ();
+
         # First, we create a cheap structure, with minimal information.
         my $message = $self->{MB_notparsed_type}->new
           ( head       => $head
@@ -368,7 +370,7 @@ sub readMessages()
           , filename   => $msgfile
           , size       => $size
           , msgnr      => $msgnr
-          , labels     => $labels[$msgnr] || undef
+          , labels     => [ seen => 1, @labels ]
           );
 
         $self->addMessage($message) if $message;
@@ -387,6 +389,13 @@ sub readMessages()
         $self->readMessage($_) foreach 0..$#msgnrs;
     }
 
+    # Which one becomes current?
+    foreach ($self->allMessages)
+    {   next unless $_->label('current') || 0;
+        $self->current($_);
+        last;
+    }
+
     $self;
 }
  
@@ -549,7 +558,10 @@ sub writeMessages()
 
         # Collect the labels.
         my $labels = $message->labels;
-        push @{$labeled{$_}}, $writer foreach keys %$labels;
+        foreach (keys %$labels)
+        {   push @{$labeled{$_}}, $writer
+                if $labels->{$_};
+        }
 
         push @{$labeled{unseen}}, $writer
             unless $labels->{seen} || 0;
@@ -833,11 +845,11 @@ sub readLabels()
     while(<SEQ>)
     {   s/\s*\#.*$//;
         next unless length;
-        my $label;
 
-        next unless ($label) = s/^\s*(\w+)\s*\:\s*//;
+        next unless s/^\s*(\w+)\s*\:\s*//;
+        my $label = $1;
 
-        my $set = 1;
+        my $set   = 1;
            if($label eq 'cur'   ) { $label = 'current' }
         elsif($label eq 'unseen') { $label = 'seen'; $set = 0 }
 
@@ -874,16 +886,14 @@ sub writeLabels($)
     my $seq      = FileHandle->new($filename, 'w') or return;
     my $oldout   = select $seq;
 
-    print "# Generated by ",__PACKAGE__,"\n";
-
     local $" = ' ';
     foreach (sort keys %$labeled)
     {
+        my @msgs  = @{$labeled->{$_}};  #they are ordered already.
         next if $_ eq 'seen';
         $_ = 'cur' if $_ eq 'current';
-
         print "$_:";
-        my @msgs  = @{$labeled->{$_}};  #they are ordered already.
+
         while(@msgs)
         {   my $start = shift @msgs;
             my $end   = $start;
@@ -893,6 +903,7 @@ sub writeLabels($)
 
             print $start==$end ? " $start" : " $start-$end";
         }
+        print "\n";
     }
 
     select $oldout;
@@ -1190,20 +1201,6 @@ will return C<undef> when the message is not read from a file.
 
 sub filename() { shift->{MBM_filename} }
 
-#-------------------------------------------
-
-=item headIsRead
-
-Checks if the head of the message is read.  This is true for fully
-parsed messages and messages where the header was accessed once.
-
-=cut
-
-sub headIsRead()
-{   my $self = shift;
-    $self->isParsed || exists $self->{MBM_head};
-}
-
 ###
 ### Mail::Box::MH::Message
 ###
@@ -1362,6 +1359,17 @@ sub head()
 
 #-------------------------------------------
 
+=item headIsRead
+
+Checks if the head of the message is read.  This is true for fully
+parsed messages and messages where the header was accessed once.
+
+=cut
+
+sub headIsRead() { exists shift->{MBM_head} }
+
+#-------------------------------------------
+
 =item messageID
 
 Retreive the message's id.  Every message has a unique message-id.  This id
